@@ -3,35 +3,34 @@ from typing import Dict, List, Optional, Sequence, cast
 
 import sqlalchemy as sa
 from aiohttp import web
-from asyncpg import Connection
-from asyncpg.exceptions import UniqueViolationError
-from sqlalchemy.sql import or_
+from sqlalchemy.exc import IntegrityError
+from sqlalchemy.sql import Select, or_
 
 from ..data.pagination import PaginatedData, Pagination
 from ..db.dbmodel import CrudDB
 from ..spec.path import ApiPath
-from ..types import DataType, Records, SchemaTypeOrStr, StrDict
-from .compile import Select, compile_query, count
+from ..types import Connection, DataType, Record, Records, SchemaTypeOrStr, StrDict
 
 unique_regex = re.compile(r"Key \((?P<column>(\w+,? ?)+)\)=\((?P<value>.+)\)")
 
 
 class SqlApiPath(ApiPath):
-    """An :class:`.ApiPath` backed by an SQL model
-    """
+    """An :class:`.ApiPath` backed by an SQL model"""
 
     table: str = ""
     """sql table name"""
 
     @property
     def db(self) -> CrudDB:
-        """Database connection pool
-        """
+        """Database connection pool"""
         return self.request.app["db"]
 
     @property
     def db_table(self) -> sa.Table:
-        """Default database table for this route"""
+        """Default database table for this route.
+
+        Obtained from the :attr:`table` attribute.
+        """
         return self.db.metadata.tables[self.table]
 
     def get_search_clause(
@@ -61,6 +60,7 @@ class SqlApiPath(ApiPath):
             is not provided
         :param table: sqlalchemy table, if not provided the default :attr:`.db_table` is
             used instead
+        :param conn: optional db connection
         """
         table = table if table is not None else self.db_table
         if filters is None:
@@ -71,8 +71,6 @@ class SqlApiPath(ApiPath):
             self.db.get_query(table, table.select(), params=filters, consumer=self),
         )
         #
-        sql_count, args_count = count(sql_query)
-        #
         # order by
         if specials["order_by"]:
             order_by_column = getattr(table.c, specials["order_by"], None)
@@ -82,11 +80,12 @@ class SqlApiPath(ApiPath):
                 sql_query = sql_query.order_by(order_by_column)
 
         # search
-        sql_query = self.get_search_clause(
+        sql = self.get_search_clause(
             table, sql_query, specials["search"], specials["search_fields"]
         )
 
         # pagination
+        sql_query = sql
         offset = specials["offset"]
         limit = specials["limit"]
         if offset:
@@ -94,13 +93,12 @@ class SqlApiPath(ApiPath):
         if limit:
             sql_query = sql_query.limit(limit)
 
-        sql, args = compile_query(sql_query)
         async with self.db.ensure_connection(conn) as conn:
-            total = await conn.fetchrow(sql_count, *args_count)
-            values = await conn.fetch(sql, *args)
+            total = await self.db.db_count_query(sql, conn=conn)
+            values = await conn.execute(sql_query)
         pagination = Pagination(self.full_url())
-        data = cast(List[StrDict], self.dump(dump_schema, values))
-        return pagination.paginated(data, total[0], offset, limit)
+        data = cast(List[StrDict], self.dump(dump_schema, values.all()))
+        return pagination.paginated(data, total, offset, limit)
 
     async def create_one(
         self,
@@ -116,19 +114,21 @@ class SqlApiPath(ApiPath):
         :param data: input data, if not given it loads it via :meth:`.json_data`
         :param table: sqlalchemy table, if not given it uses the
             default :attr:`db_table`
+        :param conn: optional db connection
         """
         if data is None:
             data = self.insert_data(await self.json_data(), body_schema=body_schema)
         table = table if table is not None else self.db_table
-        statement, args = self.db.get_insert(table, data)
+        sql = self.db.get_insert(table, data)
 
         async with self.db.ensure_connection(conn) as conn:
             try:
-                values = await conn.fetch(statement, *args)
-            except UniqueViolationError as exc:
+                result = await conn.execute(sql)
+            except IntegrityError as exc:
                 self.handle_unique_violation(exc)
+                raise
 
-        return cast(StrDict, self.dump(dump_schema, values[0]))
+        return cast(StrDict, self.dump(dump_schema, result.one()))
 
     async def create_list(
         self,
@@ -139,8 +139,7 @@ class SqlApiPath(ApiPath):
         dump_schema: SchemaTypeOrStr = "response_schema",
         conn: Optional[Connection] = None,
     ):
-        """Create multiple models
-        """
+        """Create multiple models"""
         table = table if table is not None else self.db_table
         if data is None:
             data = await self.json_data()
@@ -152,7 +151,7 @@ class SqlApiPath(ApiPath):
         assert schema.container is list
         data = [self.insert_data(d, body_schema=schema.element) for d in data]
         values = await self.db.db_insert(table, data, conn=conn)
-        return self.dump(dump_schema, values)
+        return self.dump(dump_schema, values.all())
 
     async def get_one(
         self,
@@ -165,14 +164,23 @@ class SqlApiPath(ApiPath):
         conn: Optional[Connection] = None,
     ):
         """Get a single model
+
+        :param filters: dictionary of filters, if not provided it will be created from
+            the query_schema
+        :param query: additional query parameters, only used when filters
+            is not provided
+        :param table: sqlalchemy table, if not provided the default :attr:`.db_table` is
+            used instead
+        :param conn: optional db connection
         """
         table = table if table is not None else self.db_table
         if filters is None:
             filters = self.get_filters(query=query, query_schema=query_schema)
         values = await self.db.db_select(table, filters, conn=conn, consumer=self)
-        if not values:
+        row = values.first()
+        if row is None:
             raise web.HTTPNotFound()
-        return self.dump(dump_schema, values[0])
+        return self.dump(dump_schema, row)
 
     async def update_one(
         self,
@@ -186,8 +194,7 @@ class SqlApiPath(ApiPath):
         dump_schema: SchemaTypeOrStr = "response_schema",
         conn: Optional[Connection] = None,
     ):
-        """Update a single model
-        """
+        """Update a single model"""
         table = table if table is not None else self.db_table
         if data is None:
             data = self.cleaned("body_schema", await self.json_data(), strict=False)
@@ -199,13 +206,15 @@ class SqlApiPath(ApiPath):
                 values = await self.db.db_update(
                     table, filters, data, conn=conn, consumer=self
                 )
-            except UniqueViolationError as exc:
+            except IntegrityError as exc:
                 self.handle_unique_violation(exc)
+                raise
         else:
             values = await self.db.db_select(table, filters, conn=conn, consumer=self)
-        if not values:
+        row = values.first()
+        if row is None:
             raise web.HTTPNotFound()
-        return self.dump(dump_schema, values[0])
+        return self.dump(dump_schema, row)
 
     async def delete_one(
         self,
@@ -215,16 +224,16 @@ class SqlApiPath(ApiPath):
         table: Optional[sa.Table] = None,
         query_schema: SchemaTypeOrStr = "query_schema",
         conn: Optional[Connection] = None,
-    ) -> Records:
-        """Delete a single model
-        """
+    ) -> Record:
+        """Delete a single model"""
         table = table if table is not None else self.db_table
         if filters is None:
             filters = self.get_filters(query=query, query_schema=query_schema)
         values = await self.db.db_delete(table, filters, conn=conn, consumer=self)
-        if not values:
-            raise web.HTTPNotFound
-        return values
+        row = values.first()
+        if row is None:
+            raise web.HTTPNotFound()
+        return row
 
     async def delete_list(
         self,
@@ -234,18 +243,15 @@ class SqlApiPath(ApiPath):
         table: Optional[sa.Table] = None,
         conn: Optional[Connection] = None,
     ) -> Records:
-        """delete multiple models
-        """
+        """delete multiple models"""
         table = table if table is not None else self.db_table
         if filters is None:
             filters = self.get_filters(query=query)
         return await self.db.db_delete(table, filters, conn=conn, consumer=self)
 
-    def handle_unique_violation(self, exception):
-        match = re.match(unique_regex, exception.detail)
-        if not match:  # pragma: no cover
-            raise exception
-
-        column = match.group("column")
-        message = f"{column} already exists"
-        self.raise_validation_error(message=message)
+    def handle_unique_violation(self, exception: IntegrityError):
+        match = re.search(unique_regex, str(exception))
+        if match:
+            column = match.group("column")
+            message = f"{column} already exists"
+            self.raise_validation_error(message=message)
