@@ -83,15 +83,13 @@ class ApiPath(web.View, DataView):
         except Exception:
             self.raise_bad_data()
 
-    def raise_validation_error(
+    def validation_error(
         self, message: str = "", errors: Optional[ErrorType] = None
-    ) -> None:
+    ) -> Exception:
+        """Create an :class:`aiohttp.web.HTTPUnprocessableEntity`"""
         raw = self.as_errors(message, errors)
         data = self.dump(ValidationErrors, raw)
-        raise web.HTTPUnprocessableEntity(**self.api_response_data(data))
-
-    # backward compatibility
-    raiseValidationError = raise_validation_error
+        return web.HTTPUnprocessableEntity(**self.api_response_data(data))
 
     def raise_bad_data(
         self, exc: Optional[Exception] = None, message: str = ""
